package org.xtext.example.mydsl.idea.intentions

import com.google.inject.Inject
import com.google.inject.Provider
import com.intellij.lang.annotation.Annotation
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.project.Project
import com.intellij.psi.PsiFile
import com.intellij.util.IncorrectOperationException
import org.eclipse.xtext.idea.intentions.AbstractIssueIntentionAction
import org.eclipse.xtext.idea.intentions.IdeaIntentionsProvider
import org.eclipse.xtext.validation.Issue
import org.xtext.example.mydsl.validation.MyDslValidator
import com.intellij.psi.PsiDocumentManager
import com.google.inject.Singleton

@Singleton
class MyDslIntentionsProvider extends IdeaIntentionsProvider {
	
	@Inject Provider<MyDslIntentionsProvider.CapitalizeNameIntentionAction> invalidNameIntentionActionProvider
	
	override registerFixes(Annotation anno, Issue issue) {
		if (MyDslValidator.INVALID_NAME == issue.code) {
			anno.registerFix (invalidNameIntentionActionProvider.get => [
				it.issue = issue
			])
		}
	}
	
	static class CapitalizeNameIntentionAction extends AbstractIssueIntentionAction {
		
		override getText() {
		 	"Capitalize name"
		}
		
		override invoke(Project project, Editor editor, PsiFile file) throws IncorrectOperationException {
			val firstLetter = editor.document.charsSequence.charAt(issue.offset).toString
			editor.document.replaceString(issue.offset, issue.offset + 1, firstLetter.toUpperCase)
			PsiDocumentManager.getInstance(project).commitDocument(editor.document)	
		}
		
	}
	
}