package org.xtext.example.mydsl.idea.liveTemplates

import com.google.inject.Inject
import com.intellij.codeInsight.template.TemplateContextType
import com.intellij.lang.ParserDefinition
import com.intellij.psi.PsiFile
import com.intellij.psi.tree.TokenSet
import java.util.HashSet
import java.util.concurrent.atomic.AtomicBoolean
import org.eclipse.emf.ecore.EStructuralFeature
import org.eclipse.xtext.AbstractElement
import org.eclipse.xtext.IGrammarAccess
import org.eclipse.xtext.ide.editor.contentassist.antlr.FollowElementComputer
import org.eclipse.xtext.ide.editor.contentassist.antlr.IContentAssistParser
import org.eclipse.xtext.idea.editorActions.TokenSetProvider
import org.eclipse.xtext.idea.lang.IXtextLanguage
import org.eclipse.xtend.lib.annotations.Accessors

abstract class AbstractXtextTemplateContextType extends TemplateContextType {
	
	@Inject protected IGrammarAccess grammarAccess
	@Inject protected IContentAssistParser contentAssistParser
	@Inject protected FollowElementComputer followElementComputer
	@Inject protected ParserDefinition parserDefinition
	@Inject protected TokenSetProvider tokenSetProvider
	
	protected HashSet<TokenSet> tokenSets = newHashSet
	protected HashSet<AbstractElement> followElements = newHashSet
	
	@Accessors(PROTECTED_GETTER)
	val IXtextLanguage language

	protected new(IXtextLanguage language, String id, String presentableName, Class<? extends TemplateContextType> parent) {
		super(id, presentableName, parent)
		this.language = language
		language.injectMembers(this)
	}

	protected def registerFollowElementsfor(EStructuralFeature eStructuralFeature) {
		followElementComputer.collectAbstractElements(grammarAccess.grammar, eStructuralFeature) [
			followElements += it
		]
	}

	final override isInContext(PsiFile file, int offset) {
		if (file.language !== getLanguage)
			return false;
		return internalIsInContext(file, offset)
	}

	def boolean internalIsInContext(PsiFile file, int offset) {
		if (tokenSets.empty && followElements.empty)
			return true

		if (!tokenSets.empty) {
			val tokenSet = getTokenSet(file, offset)
			if (!tokenSets.contains(tokenSet))
				return false
			if (followElements.empty)
				return true
		}

		if (followElements.empty)
			return false

		val element = file.findElementAt(offset)
		if (element?.node === null)
			return false

		val elements = contentAssistParser.getFollowElements(file.text.substring(0, element.node.startOffset), false)
		val hasFollowElement = new AtomicBoolean(false)
		followElementComputer.computeFollowElements(elements) [
			if (!hasFollowElement.get && followElements.contains(it)) {
				hasFollowElement.set(true)
			}
		]
		return hasFollowElement.get
	}
	
	protected def getTokenSet(PsiFile file, int offset) {
		val lexer = parserDefinition.createLexer(file.project)
		lexer.start(file.text)
		while (lexer.tokenType != null && lexer.tokenEnd <= offset)
			lexer.advance

		val tokenType = lexer.tokenType
		if (tokenType == null)
			return false

		tokenSetProvider.getTokenSet(tokenType)
	}

	override toString() {
		contextId
	}
	
}