package org.xtext.example.mydsl.idea.liveTemplates;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import com.intellij.codeInsight.template.TemplateContextType;
import com.intellij.lang.ASTNode;
import com.intellij.lang.Language;
import com.intellij.lang.ParserDefinition;
import com.intellij.lexer.Lexer;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.tree.IElementType;
import com.intellij.psi.tree.TokenSet;
import java.util.Collection;
import java.util.HashSet;
import java.util.concurrent.atomic.AtomicBoolean;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.xtend.lib.annotations.AccessorType;
import org.eclipse.xtend.lib.annotations.Accessors;
import org.eclipse.xtext.AbstractElement;
import org.eclipse.xtext.Grammar;
import org.eclipse.xtext.IGrammarAccess;
import org.eclipse.xtext.ide.editor.contentassist.IFollowElementAcceptor;
import org.eclipse.xtext.ide.editor.contentassist.antlr.FollowElement;
import org.eclipse.xtext.ide.editor.contentassist.antlr.FollowElementComputer;
import org.eclipse.xtext.ide.editor.contentassist.antlr.IContentAssistParser;
import org.eclipse.xtext.idea.editorActions.TokenSetProvider;
import org.eclipse.xtext.idea.lang.IXtextLanguage;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Pure;

@SuppressWarnings("all")
public abstract class AbstractXtextTemplateContextType extends TemplateContextType {
  @Inject
  protected IGrammarAccess grammarAccess;
  
  @Inject
  protected IContentAssistParser contentAssistParser;
  
  @Inject
  protected FollowElementComputer followElementComputer;
  
  @Inject
  protected ParserDefinition parserDefinition;
  
  @Inject
  protected TokenSetProvider tokenSetProvider;
  
  protected HashSet<TokenSet> tokenSets = CollectionLiterals.<TokenSet>newHashSet();
  
  protected HashSet<AbstractElement> followElements = CollectionLiterals.<AbstractElement>newHashSet();
  
  @Accessors(AccessorType.PROTECTED_GETTER)
  private final IXtextLanguage language;
  
  protected AbstractXtextTemplateContextType(final IXtextLanguage language, final String id, final String presentableName, final Class<? extends TemplateContextType> parent) {
    super(id, presentableName, parent);
    this.language = language;
    language.injectMembers(this);
  }
  
  protected void registerFollowElementsfor(final EStructuralFeature eStructuralFeature) {
    Grammar _grammar = this.grammarAccess.getGrammar();
    final IFollowElementAcceptor _function = new IFollowElementAcceptor() {
      @Override
      public void accept(final AbstractElement it) {
        AbstractXtextTemplateContextType.this.followElements.add(it);
      }
    };
    this.followElementComputer.collectAbstractElements(_grammar, eStructuralFeature, _function);
  }
  
  @Override
  public final boolean isInContext(final PsiFile file, final int offset) {
    Language _language = file.getLanguage();
    IXtextLanguage _language_1 = this.getLanguage();
    boolean _tripleNotEquals = (_language != _language_1);
    if (_tripleNotEquals) {
      return false;
    }
    return this.internalIsInContext(file, offset);
  }
  
  public boolean internalIsInContext(final PsiFile file, final int offset) {
    boolean _and = false;
    boolean _isEmpty = this.tokenSets.isEmpty();
    if (!_isEmpty) {
      _and = false;
    } else {
      boolean _isEmpty_1 = this.followElements.isEmpty();
      _and = _isEmpty_1;
    }
    if (_and) {
      return true;
    }
    boolean _isEmpty_2 = this.tokenSets.isEmpty();
    boolean _not = (!_isEmpty_2);
    if (_not) {
      final Object tokenSet = this.getTokenSet(file, offset);
      boolean _contains = this.tokenSets.contains(tokenSet);
      boolean _not_1 = (!_contains);
      if (_not_1) {
        return false;
      }
      boolean _isEmpty_3 = this.followElements.isEmpty();
      if (_isEmpty_3) {
        return true;
      }
    }
    boolean _isEmpty_4 = this.followElements.isEmpty();
    if (_isEmpty_4) {
      return false;
    }
    final PsiElement element = file.findElementAt(offset);
    ASTNode _node = null;
    if (element!=null) {
      _node=element.getNode();
    }
    boolean _tripleEquals = (_node == null);
    if (_tripleEquals) {
      return false;
    }
    String _text = file.getText();
    ASTNode _node_1 = element.getNode();
    int _startOffset = _node_1.getStartOffset();
    String _substring = _text.substring(0, _startOffset);
    final Collection<FollowElement> elements = this.contentAssistParser.getFollowElements(_substring, false);
    final AtomicBoolean hasFollowElement = new AtomicBoolean(false);
    final IFollowElementAcceptor _function = new IFollowElementAcceptor() {
      @Override
      public void accept(final AbstractElement it) {
        boolean _and = false;
        boolean _get = hasFollowElement.get();
        boolean _not = (!_get);
        if (!_not) {
          _and = false;
        } else {
          boolean _contains = AbstractXtextTemplateContextType.this.followElements.contains(it);
          _and = _contains;
        }
        if (_and) {
          hasFollowElement.set(true);
        }
      }
    };
    this.followElementComputer.computeFollowElements(elements, _function);
    return hasFollowElement.get();
  }
  
  protected Object getTokenSet(final PsiFile file, final int offset) {
    TokenSet _xblockexpression = null;
    {
      Project _project = file.getProject();
      final Lexer lexer = this.parserDefinition.createLexer(_project);
      String _text = file.getText();
      lexer.start(_text);
      while (((!Objects.equal(lexer.getTokenType(), null)) && (lexer.getTokenEnd() <= offset))) {
        lexer.advance();
      }
      final IElementType tokenType = lexer.getTokenType();
      boolean _equals = Objects.equal(tokenType, null);
      if (_equals) {
        return Boolean.valueOf(false);
      }
      _xblockexpression = this.tokenSetProvider.getTokenSet(tokenType);
    }
    return _xblockexpression;
  }
  
  @Override
  public String toString() {
    return this.getContextId();
  }
  
  @Pure
  protected IXtextLanguage getLanguage() {
    return this.language;
  }
}
